/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2004 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JonasHandler.java,v 1.10 2005/01/26 08:51:05 durieuxp Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.jonas.ws.axis;


import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.axis.AxisFault;
import org.apache.axis.Handler;
import org.apache.axis.MessageContext;
import org.apache.axis.handlers.BasicHandler;

import org.objectweb.jonas_ejb.container.JServiceEndpointHome;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.security.ws.SecurityContextHelper;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;


/**
 * This handler is called before all application specific handlers
 * It allows to process some jonas specific stuff like setting
 * the bean environment for example.
 * @author Philippe Durieux
 */
public class JonasHandler extends BasicHandler {

    /**
     * logger
     */
    private static Logger logger = Log.getLogger(Log.JONAS_WS_EJBPROVIDER_PREFIX);

    /**
     * cached initial context
     */
    private static InitialContext cachedContext;

    /**
     * saved component context
     */
    private Context savedCtx = null;

    /**
     * ws home
     */
    private JServiceEndpointHome sehome = null;

    /**
     * Called on reply if a fault occured.
     * @param msgContext MessageContext
     */
    public void onFault(MessageContext msgContext) {
        logger.log(BasicLevel.DEBUG, "*------* Fault");
        if (sehome != null) {
            logger.log(BasicLevel.DEBUG, "*--* Fault");
            sehome.resetCompCtx(savedCtx);
            if (savedCtx != null) {
                logger.log(BasicLevel.ERROR, "Setting a non null context:" + savedCtx);
            }
            sehome = null;
        }
    }

    /**
     * Called on request and then on reply if no fault occured.
     * @param msgContext MessageContext
     * @throws AxisFault if the handler cannot get the service endpoint home
     */
    public void invoke(MessageContext msgContext) throws AxisFault {

        if (msgContext.getPastPivot()) {
            logger.log(BasicLevel.DEBUG, "*------* Response");
            // Response
            if (sehome != null) {
                logger.log(BasicLevel.DEBUG, "*--* Response");
                sehome.resetCompCtx(savedCtx);
                if (savedCtx != null) {
                    logger.log(BasicLevel.ERROR, "Setting a non null context:" + savedCtx);
                }
                sehome = null;
            }
        } else {
            // Request
            logger.log(BasicLevel.DEBUG, "*------* Request");

            // Get the service class name
            Handler service = msgContext.getService();
            String clsName = (String) service.getOption(JOnASEJBProvider.OPTION_SEINTERFACENAME);
            if (clsName == null) {
                // We are not in the case of jonas ejb call: do nothing.
                return;
            }
            logger.log(BasicLevel.DEBUG, "*--* Request");

            // Get ServiceEndpointHome in JNDI
            String jndiName = (String) service.getOption(JOnASEJBProvider.OPTION_SEJNDINAME);
            if (jndiName == null) {
                logger.log(BasicLevel.ERROR, "Service Endpoint JNDI name is null");
                throw new AxisFault("Missing parameter OPTION_SEJNDINAME in service");
            }
            try {
                InitialContext ic = getCachedContext();
                sehome = (JServiceEndpointHome) ic.lookup(jndiName);
            } catch (NamingException ne) {
                logger.log(BasicLevel.ERROR, "Cannot lookup ServiceEndpointHome");
                throw new AxisFault("Cannot lookup ServiceEndpointHome: " + jndiName);
            }
            
            // add the security context
            String username = msgContext.getUsername();
            if (username != null) {
                // Do not forget to initialize the security context
                SecurityContextHelper.getInstance().login(username, msgContext.getPassword());
                // Check the security and throw exception if user is not authorized
                // to access the EJB before lauching handlers
                sehome.checkSecurity(msgContext);
            }
            
            // Switch Context on the context of the target bean (java:comp/env)
            savedCtx = sehome.setCompCtx();
            if (savedCtx != null) {
                logger.log(BasicLevel.ERROR, "Saving a non null context:" + savedCtx);
            }

        }
    }

    /**
     * @return Returns the cached InitialContext (or created a new one)
     * @throws javax.naming.NamingException when InitialContext creation fails
     */
    private InitialContext getCachedContext() throws javax.naming.NamingException {
        if (cachedContext == null) {
            cachedContext = new InitialContext();
        }
        return cachedContext;
    }
}
